local SML = AceLibrary("SharedMedia-1.0")
local BC = AceLibrary("Babble-Class-2.2")

local max_unit_size = 5
local header_padding = 33

local classIcons = {
	["Warrior"] = {0, 0.25, 0, 0.25},
	["Mage"] = {0.25, 0.49609375, 0, 0.25},
	["Rogue"] = {0.49609375, 0.7421875, 0, 0.25},
	["Druid"] = {0.7421875, 0.98828125, 0, 0.25},
	["Hunter"] = {0, 0.25, 0.25, 0.5},
	["Shaman"] = {0.25, 0.49609375, 0.25, 0.5},
	["Priest"] = {0.49609375, 0.7421875, 0.25, 0.5},
	["Warlock"] = {0.7421875, 0.98828125, 0.25, 0.5},
	["Paladin"] = {0, 0.25, 0.5, 0.75},
}

function Proximo:MakeFrame()
	local barwidth = self.db.profile.barwidth
	local barheight = self.db.profile.barheight
	local padding = self.db.profile.padding
	
	self.frame=CreateFrame("Button", nil, UIParent)
	self.frame.owner = self
	self.frame:Hide()
	self.frame:SetWidth(barwidth+barheight+(padding*2)+8)  
	self.frame:SetHeight(((barheight+padding)*(max_unit_size))+4+header_padding+padding) 
	self.frame:SetScale(self.db.profile.scale)
	self.frame:ClearAllPoints()
	if self.db.profile.x==0 and self.db.profile.y==0 then
		self.frame:SetPoint("TOP", UIParent, "TOP")
	else
		self.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.db.profile.x, self.db.profile.y)
	end
	self.frame:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\AddOns\\Proximo\\Textures\\otravi-semi-full-border", edgeSize = 32,
		insets = {left = 1, right = 1, top = 30, bottom = 1},
	})
	self.frame:SetBackdropColor(self.db.profile.framecolor.r,self.db.profile.framecolor.g,self.db.profile.framecolor.b,self.db.profile.framecolor.a)
	self.frame:SetBackdropBorderColor(1,1,1,0)
	self.frame:EnableMouse(true)
	self.frame:SetMovable(true)
	self.frame:RegisterForDrag("LeftButton")
	self.frame:RegisterForClicks("RightButtonUp")
	self.frame:SetScript(
		'OnClick', function()
		   self:DewDropMenu()
		end
	)
	self.frame:SetScript(
		'OnDragStart', function()
			if not InCombatLockdown() and IsAltKeyDown() then this:StartMoving() end
		end
	)
	self.frame:SetScript(
		'OnDragStop', function()
				if not InCombatLockdown() then
				this:StopMovingOrSizing()
				local _, _, _, x, y = this:GetPoint()
				this.owner.db.profile.x = x
				this.owner.db.profile.y = y
			end
		end
	)
	
	self.frame.text = self.frame:CreateFontString(nil, "OVERLAY")
	self.frame.text:ClearAllPoints()
	self.frame.text:SetWidth(barwidth)
	self.frame.text:SetHeight(header_padding)
	self.frame.text:SetPoint("TOP", self.frame, "TOP", 0, -5)
	self.frame.text:SetFont(SML:Fetch("font", self.db.profile.font), 16)
	self.frame.text:SetJustifyH("CENTER")
	self.frame.text:SetText("")
	self.frame.text:SetShadowOffset(1, -1)
	self.frame.text:SetShadowColor(0, 0, 0, 1)
	
	--set up header settings
	if (self.db.profile.header) then
		self.frame:SetBackdropBorderColor(1,1,1,1)
		self.frame.text:SetText(self.title)
	end
	
	self.frame.buttons={}
	for i=1,max_unit_size do
		local b=CreateFrame("Button", "Target", self.frame, "SecureActionButtonTemplate")
		b:SetHeight(barheight)
		b:SetWidth(barwidth)
		b:ClearAllPoints()
		if i==1 then
			b:SetPoint("TOP",self.frame,"TOP", barheight/2, -padding-header_padding)
		else
			b:SetPoint("TOP",self.frame.buttons[i-1],"BOTTOM", 0, -padding)
		end
		b:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		b:SetAttribute("*type*", "macro")
		b:SetAttribute("macrotext1", "/assist "..self.db.profile.ma)
		b:SetAttribute("macrotext2", "/target focus")
		
		b.healthBar = CreateFrame("StatusBar", "HealthBar", b)
		b.healthBar:ClearAllPoints()
		b.healthBar:SetAllPoints(b)
		b.healthBar:SetStatusBarTexture(SML:Fetch('statusbar', self.db.profile.texture))
		b.healthBar:SetMinMaxValues(0, 100)
		b.healthBar:SetValue(0)
		
		b.healthBar.highlight = CreateFrame("Frame", "Highlight", b.healthBar)
		b.healthBar.highlight:SetHeight(barheight+8)
		b.healthBar.highlight:SetWidth(barwidth+8)
		b.healthBar.highlight:ClearAllPoints()
		b.healthBar.highlight:SetPoint("TOP",b.healthBar,"TOP", 0, 4)
		b.healthBar.highlight:SetBackdrop({
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
			insets = {left = 4, right = 4, top = 4, bottom = 4},
		})
		b.healthBar.highlight:SetBackdropBorderColor(1,1,0,0)
		
		b.text = b.healthBar:CreateFontString(nil,"LOW")
		b.text:SetFont(SML:Fetch("font", self.db.profile.font), self.db.profile.fontsize)
		b.text:SetShadowOffset(1, -1)
	  b.text:SetShadowColor(0, 0, 0, 1)
		b.text:SetJustifyH("LEFT")
		b.text:ClearAllPoints()
		b.text:SetAllPoints(b.healthBar)
		
		b.healthtext = b.healthBar:CreateFontString(nil,"LOW")
		b.healthtext:SetFont(SML:Fetch("font", self.db.profile.font), self.db.profile.fontsize)
		b.healthtext:SetShadowOffset(1, -1)
	  b.healthtext:SetShadowColor(0, 0, 0, 1)
		b.healthtext:SetJustifyH("RIGHT")
		b.healthtext:ClearAllPoints()
		b.healthtext:SetAllPoints(b.healthBar)
		
		b.icon = b:CreateTexture(nil, "ARTWORK")
		b.icon:ClearAllPoints()
		b.icon:SetHeight(barheight)
		b.icon:SetWidth(barheight)
		b.icon:SetPoint("TOPRIGHT",b,"TOPLEFT",-1,0)
		
		table.insert(self.frame.buttons,b)
	end
end


function Proximo:UpdateFrame()
	local barwidth = self.db.profile.barwidth
	local barheight = self.db.profile.barheight
	local padding = self.db.profile.padding
	
	self.frame:SetWidth(barwidth+barheight+(padding*2)+8)  
	self.frame:SetHeight(((barheight+padding)*(max_unit_size))+4+header_padding+padding) 
	self.frame:SetScale(self.db.profile.scale)
	self.frame:SetBackdropColor(self.db.profile.framecolor.r,self.db.profile.framecolor.g,self.db.profile.framecolor.b,self.db.profile.framecolor.a)
	self.frame:SetBackdropBorderColor(1,1,1,0)
		
	self.frame.text:SetFont(SML:Fetch("font", self.db.profile.font), 16)
	self.frame.text:SetWidth(barwidth)  
	self.frame.text:SetText("")
	
	if (self.db.profile.header) then
		self.frame:SetBackdropBorderColor(1,1,1,1)
		self.frame.text:SetText(self.title)
	end
	
	for i=1,max_unit_size do
		local b=self.frame.buttons[i]
		b:SetHeight(barheight)
		b:SetWidth(barwidth)
		b:ClearAllPoints()
		if i==1 then
			b:SetPoint("TOP",self.frame,"TOP", barheight/2, -padding-header_padding)
		else
			b:SetPoint("TOP",self.frame.buttons[i-1],"BOTTOM", 0, -padding)
		end		
		b.healthBar:SetStatusBarTexture(SML:Fetch('statusbar', self.db.profile.texture))
		b.healthBar.highlight:SetHeight(barheight+8)
		b.healthBar.highlight:SetWidth(barwidth+8)
		b.text:SetFont(SML:Fetch("font", self.db.profile.font), self.db.profile.fontsize)
		b.healthtext:SetFont(SML:Fetch("font", self.db.profile.font), self.db.profile.fontsize)
		b.icon:SetHeight(barheight)
		b.icon:SetWidth(barheight)
	end
end

function Proximo:FrameClean()
	if not self.frame then return end
	--clean up all unit info
  for i=1,max_unit_size do
  	if self.frame.buttons[i] then
  		self.frame.buttons[i]:SetAttribute("macrotext1", "/assist "..self.db.profile.ma)
  		self.frame.buttons[i]:SetAttribute("macrotext2", "/target focus")
  		self.frame.buttons[i].healthBar:SetValue(0)
  		self.frame.buttons[i].healthtext:SetText(nil)
  		self.frame.buttons[i].text:SetText(nil)
  		self.frame.buttons[i].icon:SetTexture(nil)
  	end
  end
end

function Proximo:FrameClass(frame,class)
	local r,g,b = BC:GetColor(class)
	frame.healthBar:SetStatusBarColor(r,g,b,1)
	self:GetClassIcon(frame.icon, class)
end

function Proximo:GetClassIcon(texture, classname)
	local class = classIcons[BC:GetReverseTranslation(classname)]
	if class then
		texture:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes")
		texture:SetTexCoord(class[1], class[2], class[3], class[4])
	end
end

function Proximo:Test()
	self:ResetList()
	self:AddToList("Perseph", "Rogue", 100)
	self:AddToList("Reznic", "Warlock", 76)
	self:AddToList("Suun", "Priest", 54)
	self:AddToList("Alynn", "Paladin", 12)
	self:AddToList("Grayhoof", "Shaman", 59)
	self:FrameRefresh() 
end

----------------------
--Send Annoucement
function Proximo:SendAnnouncement(name, class)
	if self.db.profile.announce == "disabled" then 
		return
	end
	local r,g,b = BC:GetColor(class)
	if self.db.profile.announce == "chat" then
		self:Print("|cff"..BC:GetHexColor(class)..name.." ("..class..")|r")
	elseif self.db.profile.announce == "sct" and SCT then
		SCT:DisplayCustomEvent(name.." ("..class..")", {r=r,g=g,b=b}, 1, 1, 1)
	elseif self.db.profile.announce == "msbt" and MikSBT then
		MikSBT.DisplayMessage(name.." ("..class..")", MikSBT.DISPLAYTYPE_NOTIFICATION, true, r*255,g*255,b*255)
	else
		RaidWarningFrame:SetTimeVisible(3)
		RaidWarningFrame:SetHeight(120)
		RaidWarningFrame:AddMessage(name.." ("..class..")", r,g,b)
	end
end